/*
  *****************************************************************************

  Copyright 1993 The Regents of the University of California.
  All Rights Reserved.

  Permission to use, copy, modify and distribute this software and its
  documentation for educational, research and non-profit purposes, without
  fee, and without a written agreement is hereby granted, provided that the 
  above copyright notice and the following three paragraphs appear in all 
  copies.

  Permission to incorporate this software into commercial products may be
  obtained from the Office of Technology Licensing, 2150 Shattuck Avenue,
  Suite 510, Berkeley, CA 94720-1620.

  IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
  LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
  OF SUCH DAMAGES.

  THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
  ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS
  TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

  The author may be contacted via:

  US Mail:  Brian Mirtich
            387 Soda Hall
	    Computer Science Division
	    Department of Electrical Engineering and Computer Science
	    University of California
	    Berkeley, CA 94720

 Phone:     (510) 642-8149

 EMail:     mirtich@cs.berkeley.edu

 *****************************************************************************
*/



#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "cf.h"



void main()
{
  POLYHEDRON *p1, *p2;
  void *feat1, *feat2;
  REAL angles[3];
  REAL dist;
  MAT4 T;
  char name1[20], name2[20];
  int deg;

  /* initialize the polyhedron library */
  loadPolyhedronLibrary("poly.dat");

  /* instantiate two polyhedra */
  p1 = createPolyhedron("slab", "slab-1");
  p2 = createPolyhedron("pyramid", "pyramid-1");

  /*
    Uncomment the following lines to see detailed information about the
    internal strucutre of the polyhedra and voronoi regions.

  dumpPolyhedron(p1);
  dumpPolyhedron(p2);
  dumpCones(p1);
  dumpCones(p2);
  */

  printf("polyhedron 1: %-20s (%d features)\n", p1->name, numFeats(p1));
  printf("polyhedron 2: %-20s (%d features)\n", p2->name, numFeats(p2));

  /* 
    To test the closest features algorithm, we will place polyhedron p1 at
    the origin, and polyhedron p2 at the point (20, 0, 0).  Then we will
    run the distance algorithm as p1 rotates around its z-axis while p2
    rotates about its y-axis.
  */

  /* set initial poses */
  matBuildXform("", angles, 0.0, 0.0, 0.0, T);
  setPose(p1, T);
  matBuildXform("", angles, 20.0, 0.0, 0.0, T);
  setPose(p2, T);

  /* Do the initial closest features computation; we must call
     closestFeaturesInit() the first time. */
  dist = closestFeaturesInit(p1, &feat1, p2, &feat2);
  featureName(feat1, name1);
  featureName(feat2, name2);
  printf("\ninitial:    dist=%6.3f    feat1=%-15s   feat2=%-15s\n",
	 dist, name1, name2);

  /* repeatedly compute closest features as polyhedra rotate in 
     5 degree steps */
  for (deg = 0; deg < 360; deg += 5) {

    angles[0] = (REAL) deg;
    matBuildXform("z", angles, 0.0, 0.0, 0.0, T);
    setPose(p1, T);
    matBuildXform("y", angles, 20.0, 0.0, 0.0, T);
    setPose(p2, T);

    dist = closestFeatures(p1, &feat1, p2, &feat2);
    featureName(feat1, name1);
    featureName(feat2, name2);
    printf("angle=%3d:  dist=%6.3f    feat1=%-15s   feat2=%-15s\n",
	   deg, dist, name1, name2);
  }
}
